/*
 * This file implements the bodies of the members of the LinkedIntList class.
 * See LinkedIntList.h for the class declaration.
 */

#include "linkedintlist.h"
#include <iostream>
using namespace std;

// This constructor initializes a new empty list. O(1).
LinkedIntList::LinkedIntList() {
    _front = nullptr;
}

// This destructor frees up the memory used by the list. O(N).
LinkedIntList::~LinkedIntList() {
    // TODO
}

// Appends the given value to the end of the list. O(N).
void LinkedIntList::addBack(int value) {
    // TODO
}

// Inserts the given value at the front of the list. O(1).
void LinkedIntList::addFront(int value) {
    // TODO
}

// Returns value in list at given index. O(N).
// Assumes 0 <= index < size()
int LinkedIntList::get(int index) const {
    ListNode* curr = _front;
    for (int i = 0; i < index; i++) {
        curr = curr->next;
    }
    return curr->data;
}

// Inserts the given value at the given index. O(N).
// Assumes 0 <= index <= size.
void LinkedIntList::insert(int index, int value) {
    ListNode* newNode = new ListNode(value);
    if (index == 0) {
        // insert at front
        newNode->next = _front;
        _front = newNode;
    } else {
        // walk to just before the given index (James Bond)
        ListNode* curr = _front;
        for (int i = 0; i < index - 1; i++) {
            curr = curr->next;
        }

        // insert the node just after this position
        newNode->next = curr->next;
        curr->next = newNode;
    }
}

// Returns true if the list contains no elements. O(1).
bool LinkedIntList::isEmpty() const {
    return _front == nullptr;
}

// Removes the last element in the list. O(N).
void LinkedIntList::removeBack() {
    if (_front == nullptr) {
        // do nothing
    } else if (_front->next == nullptr) {
        // remove front (only) node
        delete _front;
        _front = nullptr;
    } else {
        // walk to end and delete last node
        ListNode* curr = _front;
        while (curr->next != nullptr) {
            curr = curr->next;
        }
        delete curr->next;
        curr->next = nullptr;
    }
}

// Removes the first element in the list. O(1).
void LinkedIntList::removeFront() {
    if (_front != nullptr) {
        ListNode* temp = _front->next;
        delete _front;
        _front = temp;
    }
}

// Removes value at given index from list. O(N).
// Assumes that 0 <= index < size.
void LinkedIntList::remove(int index) {
    ListNode* meghyn;
    if (index == 0) {
        meghyn = _front;
        _front = _front->next;
    } else {
        ListNode* curr = _front;
        for (int i = 0; i < index - 1; i++) {
            curr = curr->next;
        }
        meghyn = curr->next;
        curr->next = curr->next->next;
    }
    delete meghyn;
}

// Sets the element at the given index to store the given value. O(N).
// Assumes that 0 <= index < size.
void LinkedIntList::set(int index, int value) {
    ListNode* curr = _front;
    for (int i = 0; i < index; i++) {
        curr = curr->next;
    }
    curr->data = value;
}

// Returns the number of elements in the list. O(N).
int LinkedIntList::size() const {
    int count = 0;
    ListNode* curr = _front;
    while (curr != nullptr) {
        curr = curr->next;
        count++;
    }
    return count;
}

// Prints the linked list to the given output stream.
ostream& operator <<(ostream& out, LinkedIntList& list) {
    out << "{";
    if (!list.isEmpty()) {
        out << list._front->data;
        ListNode* curr = list._front->next;
        while (curr != nullptr) {
            out << ", " << curr->data;
            curr = curr->next;
        }
    }
    out << "}";
    return out;
}
